#
# Copyright (c) 2000 Shalabh Chaturvedi
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__doc__ = "iTrack Product Module"
__version__ = '0.2.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from OFS.ObjectManager import ObjectManager
from DateTime import DateTime
from DocumentTemplate import DT_Util

import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import string


manage_addIssueForm = HTMLFile('IssueAdd', globals())


def manage_addIssue(self,id,title='',details='',type='',status='',assignedTo='', parentIssueId='', REQUEST=None, creator=''):
	"""Add a new Issue object to this folder"""
	if REQUEST is not None:
		creator=REQUEST['AUTHENTICATED_USER']
	self._setObject(id, Issue(id,creator,title,details,type,status,assignedTo,parentIssueId))
	try:
		self.iTrackConfig.incrementIdUpCounter()
	except:
		pass
	if REQUEST is not None:
		if REQUEST.has_key('goto'):
			return MessageDialog(
				title="Added",
				message="A New Issue has been Created",
				action=REQUEST['goto']
				)
		else:
			return self.manage_main(self, REQUEST)


manage_addiTrackConfigForm = HTMLFile('iTrackConfigAdd', globals())



def manage_addiTrackConfig(self,title='',REQUEST=None):
	"""Add a new iTrackConfig object to this folder"""
	id = 'iTrackConfig'
	self._setObject(id, iTrackConfig(id,title))
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class Issue (
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager
	):
		meta_type="iTrack Issue"
		
		manage_options = (
			{'label': 'Basic',	 'action': 'manage_main'},
			{'label': 'Add Followup','action': 'manage_addFollowupForm'},
			{'label': 'View',	 'action': 'index_html'},
			)

		__ac_permissions__=(
			('View management screens', ('manage_tabs', 'manage_main', 'manage_addFollowupForm','manage_change','manage_addFollowup')),
			('View',      ('index_html',))
			)
#		states = ['open','closed']
			
#		_properties = (         {'id':'title',		'type':'string'},
#					{'id':'status',		'type':'selection',		'select_variable':'self.aq_parent.iTrackConfig.states'},
#					{'id':'assignedTo',	'type':'multiple selection',	'select_variable':'xyz'},
#					{'id':'blockedIssueIds','type':'multiple selection',	'select_variable':'self.states'}
#					)
			
	
		def __init__(self,id,creator,title='',details='',tipe='',status='',assignedTo=[],parentIssueId=''):
			self.id = id
			self.creator = creator
			self.dateCreated = DateTime()
			self.title = title
			self.details = details
			self.type = tipe
			self.status = status
			if type(assignedTo) == type(''):
				assignedTo = list(assignedTo)
			self.assignedTo = assignedTo
			self.parentIssueId = parentIssueId
			self.changes = []
			self.followups = []
			self.childIssueIds = []
			
		def manage_afterAdd(self, item, container):
			"afterAdd"
			self.container = container
			
			if self.parentIssueId!='None':
				parent = container._getOb(self.parentIssueId)
				parent.addChild(self.id)
				
			
		def manage_beforeDel(self, item, container):
			"beforeDel: Delete the related issuefollowups - TBD"
			if self.parentIssueId!='None':
				parent = container._getOb(self.parentIssueId)
				parent.removeChild(self.id)
			
			for child in self.childIssueIds:
				child.parentIssueId=self.parentIssueId
				
			#for followups in 
				
			

			
		def manage_addFollowup(self,id,details='',REQUEST=None,submitter=''):
		        "Add a Followup for this Issue"
			if REQUEST is not None:
				submitter=REQUEST['AUTHENTICATED_USER']
			self.container._setObject(id, IssueFollowup(id,submitter,self.id,details))
			self.changes.append({'type':'followup','followup':self.container._getOb(id)})
			self.followups.append(id)
			
			self._p_changed=1
			if REQUEST is not None:
				if REQUEST.has_key('goto'):
					return MessageDialog(
						title="Added",
						message="Followup has been added to the Issue",
						action=REQUEST['goto']
						)
				else:
					return self.manage_main(self, REQUEST)
					

				
		def manage_change(self, assignedTo, status, REQUEST=None):
		        "Change some properties for this Issue"
		        propertychanges=[]
			if status!=self.status:
				propertychanges.append(('status',self.status,status))  #(what, oldvalue, newvalue)
				self.status=status
			if type(assignedTo)==type('string'):
				assignedTo=list(assignedTo)
			if assignedTo!=self.assignedTo:
				propertychanges.append(('assignedTo',self.assignedTo,assignedTo))
				self.assignedTo=assignedTo
				
			self._p_changed=1
			
			message="No Change to Properties"
			if len(propertychanges)>0:
				if REQUEST is not None:
					submitter=REQUEST['AUTHENTICATED_USER']
				else:
					submitter='anonymous'
				dateSubmitted = DateTime()
				
				self.changes.append({'type'			:'change',
							'propertychanges'	:propertychanges,
							'submitter'		:submitter,
							'dateSubmitted'		:dateSubmitted})
				message="Properties Changed"
							
			
			if REQUEST is not None:
				if REQUEST.has_key('goto'):
					return MessageDialog(
						title="Added",
						message=message,
						action=REQUEST['goto']
						)
				else:
					return self.manage_main(self, REQUEST)
					

				
		def getNumFollowups(self):
			"Returns the number of followups submitted"
			return len(self.followups)
			
		def getChangesTrail(self):
			"Returs a list of changes on this issue (including followups)"
			return self.changes
			
		def getFollowups(self):
			"Returns a list of all followups to this issue"
			return self.followups
			
		def getDetailsHTMLQuote(self):
			"Returns the details after html_quote"
			return DT_Util.html_quote(self.details)
			
		def getChildren(self):
			"Returns the list of child issue _objects_"
			return map(lambda id, con=self.container: con._getOb(id), self.childIssueIds)

		def addChild(self, id):
			"Add an id as a child issue of this issue"
			self.childIssueIds.append(id)
			return
		
		manage_main = HTMLFile('IssueMain',globals())
		manage_addFollowupForm = HTMLFile('AddIssueFollowupForm',globals())
		index_html = HTMLFile('IssueIndex',globals())
		

class IssueFollowup (
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager
	):
		meta_type="iTrack IssueFollowup"
		
		icon="misc_/iTrack/followup_icon.gif"
			
		def __init__(self,id,submitter,forIssueId,details=''):
			self.id = id
			self.submitter = submitter
			self.dateSubmitted = DateTime()
			self.forIssueId = forIssueId
			self.details = details
			
		def getDetailsHTMLQuote(self):
			"Returns the details after html_quote"
			return DT_Util.html_quote(self.details)
			

class iTrackConfig (
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager
	):
		"Singleton in a folder. Manages config for all isses etc in that folder"
		
		meta_type="iTrackConfig"
		
	
		color2codes = { 'LtGrey' : '#F0F0F0',
				'Grey'	   : 'grey',
				'Silver'   : 'silver',
				'Blue'     : 'blue',
				'Red'	   : 'red',
				'Orange'   : 'orange',
				'Green'    : 'green',
				'Black'    : 'black'
				}

		
		manage_options = ({'label' : 'Configuration', 'action' : 'manage_main'},
				  {'label' : 'Issue View Preferences'   , 'action' : 'manage_changePresentationForm'},
				  {'label' : 'View'         , 'action' : 'index_html'}) 
		
		states = ['open','closed']
		types = ['WorkItem','Idea','Discussion']
		
		def __init__(self,id,title):
			self.id = id
			self.title = title
			self.idPrefix = 'issue-'
			self.upCounter=1
			self.insertTextMethod=None
			self.insertTextMethodName=''
			
		

		manage_main = HTMLFile('iTrackConfigMain',globals())
		manage_changePresentationForm = HTMLFile('iTrackConfigPresentation',globals())
		index_html = HTMLFile('iTrackConfigIndex',globals())
		
		def manage_changeproperties(self, states, types, idPrefix, upCounter=1, REQUEST=None):
			"Change properties of this object"
			self.states = string.split((DT_Util.html_quote(states)))
			self.types = string.split((DT_Util.html_quote(types)))
			self.idPrefix = idPrefix
			self.upCounter = upCounter
			self._p_changed=1
			
			if REQUEST is not None:
				return self.manage_main(self, REQUEST)
				
		def manage_changepresentation(self, insertTextMethodName, labelBgColor='', textBgColor='', labelColor='', textColor='', REQUEST=None):
			"Change presentation properties of the objects"
			if insertTextMethodName:
				insertTextMethod=self.aq_parent._getOb(insertTextMethodName)
				self.insertTextMethod = insertTextMethod
				self.insertTextMethodName = insertTextMethodName
			else:
				self.insertTextMethod = None
				self.insertTextMethodName = ''
			self.labelBgColor = labelBgColor
			self.textBgColor = textBgColor
			self.labelColor = labelColor
			self.textColor = textColor
		
			if REQUEST is not None:
				return self.manage_changePresentationForm(self, REQUEST)
			
		def suggestIdForNewIssue(self):
			return self.idPrefix + str(self.upCounter)
		
		def incrementIdUpCounter(self):
			self.upCounter = self.upCounter+1
			return
			
#		def doFilterIssues(self,issuesList, filter, container=None):
#			""" Filters a list of issues based on a 'filter'. A 'filter' is a tuple (condition, attribute, value)."""
#			s = []
#			
#			
#			for issue in issuesList:
#				issueDict = issue.__dict__
#				if issueDict.has_key(filter[1]):
#					if self.__class__.__dict__.has_key('testOperator_'+
#					if self.testCondition(issueDict[filter[2]],
#					exe
#					
#				else:
#					continue
			
#		def getIssues(self,filter=[],groupby=[],container=None):
#			if type(filter)==type(''):
#				filter  = list(filter)
#			if type(groupby)==type(''):
#				groupby = list(groupby)
#			
#			if container is None:
#				container = aq_parent
#				
#			s={}
#			
#			for issue in container.objectValues('iTrack Issue'):
#							
		def testOperator_EqualTo(self, a, b):
			return a==b
			
		def testOperator_LessThan(self, a, b):
			return a<b
			
		def testOperator_GreaterThan(self, a,b):
			return a>b
			
		def testOperator_Present(self, a, b=None):
			return a=='' or a==None
			
		def testOperator_NotPresent(self, a, b=None):
			return not self.testOperator_Present(a)